; ---------------------------------------------------------------------------
; Supergrafx hardware definition / (c) SyX 2013
; ---------------------------------------------------------------------------

; ---------------------------------------------------------------------------
; CTC
; ---------------------------------------------------------------------------
; I/O Ports
CTC_TIM0            EQU $F880   ; Channel 0 (I: 4 MHz clock | O: YMZ clock)
CTC_TIM1            EQU $F881   ; Channel 1 (I: Cursor CRTC | O: NMI)
CTC_TIM2            EQU $F882   ; Channel 2 (I: 4 MHz clock | O: CTC_TIM3 TRG)
CTC_TIM3            EQU $F883   ; Channel 3 (I: CTC_TIM2 TO)

; ---------------------------------------------------------------------------
; D0-D7 : Data bus (R/W) Used for communicating with the Z80
; ---------------------------------------------------------------------------
; Control Word
; ------------
; D7: Interrupt (0: Disable / 1: Enable)
; D6: Mode (0: Timer / 1: Counter)
; D5: Prescaler Value (0: 16 / 1: 256) *** Only in Timer mode
; D4: Clock/Trigger Edge Selection (0: Falling Edge / 1: Rising Edge)
; D3: Timer Trigger (0: Automatic Trigger when time constant is loaded 
;                    1: CLK/TRG pulse starts timer) *** Only in Timer mode
; D2: Time Constant (0: No Time Constant / 1: Time Constant follows)
; D1: Reset (0: Continued Operation / 1: Software Reset)
; D0: Control or Vector (0: Vector / 1: Control Word)
; ---------------------------------------------------------------------------
; Constants for Control Word
CTC_NO_INT          EQU %00000000
CTC_INT             EQU %10000000
CTC_USE_TIMER       EQU %00000000
CTC_USE_COUNTER     EQU %01000000
CTC_PRESCALER_16    EQU %00000000
CTC_PRESCALER_256   EQU %00100000
CTC_FALLING_EDGE    EQU %00000000
CTC_RISING_EDGE     EQU %00010000
CTC_AUTO_TRIGGER    EQU %00000000
CTC_CLK_TRIGGER     EQU %00001000
CTC_NO_CONSTANT     EQU %00000000
CTC_CONSTANT        EQU %00000100
CTC_NO_RESET        EQU %00000000
CTC_RESET           EQU %00000010
CTC_VECTOR          EQU %00000000
CTC_CONTROL         EQU %00000001

; Typical values
CTC_START_TIMER16   EQU CTC_NO_INT|CTC_USE_TIMER|CTC_PRESCALER_16|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_TIMER256  EQU CTC_NO_INT|CTC_USE_TIMER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ITIMER16  EQU CTC_INT|CTC_USE_TIMER|CTC_PRESCALER_16|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ITIMER256 EQU CTC_INT|CTC_USE_TIMER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_COUNTER   EQU CTC_NO_INT|CTC_USE_COUNTER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_START_ICOUNTER  EQU CTC_INT|CTC_USE_COUNTER|CTC_PRESCALER_256|CTC_RISING_EDGE|CTC_AUTO_TRIGGER|CTC_CONSTANT|CTC_RESET|CTC_CONTROL
CTC_STOP_CHANNEL    EQU CTC_RESET|CTC_CONTROL
    
; ---------------------------------------------------------------------------
; YMZ
; ---------------------------------------------------------------------------
; I/O Ports
YMZ_SELECT_LEFT     EQU $F984   ; Select register (Left YMZ)
YMZ_WRITE_LEFT      EQU $F884   ; Write in the selected register (Left YMZ)
YMZ_SELECT_RIGHT    EQU $F988   ; Select register (Right YMZ)
YMZ_WRITE_RIGHT     EQU $F888   ; Write in the selected register (Right YMZ)

; ---------------------------------------------------------------------------
; YMZ Registers
YMZ_REG_00          EQU $00     ; Channel A Frequency (8 Bit Fine Tune)
YMZ_REG_01          EQU $01     ; Channel A Frequency (4 Bit Coarse Tune)
YMZ_REG_02          EQU $02     ; Channel B Frequency (8 Bit Fine Tune)
YMZ_REG_03          EQU $03     ; Channel B Frequency (4 Bit Coarse Tune)
YMZ_REG_04          EQU $04     ; Channel C Frequency (8 Bit Fine Tune)
YMZ_REG_05          EQU $05     ; Channel C Frequency (4 Bit Coarse Tune)
YMZ_REG_06          EQU $06     ; Noise Frequency (5 Bit Period Control)
YMZ_REG_07          EQU $07     ; Mixer Setting (Noise CBA + Tone CBA) 
YMZ_REG_08          EQU $08     ; Channel A Volume (Envelope + 4 Bits)
YMZ_REG_09          EQU $09     ; Channel B Volume (Envelope + 4 Bits)
YMZ_REG_0A          EQU $0A     ; Channel c Volume (Envelope + 4 Bits)
YMZ_REG_0B          EQU $0B     ; Envelope Frequency (8 Bit Fine Tune)
YMZ_REG_0C          EQU $0C     ; Envelope Frequency (8 Bit Coarse Tune)
YMZ_REG_0D          EQU $0D     ; Envelope Shape (4 Bits)
YMZ_REG_0F          EQU $0F     ; Control Power (%0000----)

; ---------------------------------------------------------------------------
